// ********************************************************************/
// Actel Corporation Proprietary and Confidential
// Copyright 2010 Actel Corporation.  All rights reserved.
//
// ANY USE OR REDISTRIBUTION IN PART OR IN WHOLE MUST BE HANDLED IN
// ACCORDANCE WITH THE ACTEL LICENSE AGREEMENT AND MUST BE APPROVED
// IN ADVANCE IN WRITING.
//
// Description: FIFO pointer and flag logic for CoreSDLC - requires 
//				two-port RAM, or equivalent in logic gates
//
// Revision Information:
// Date			Description
// ----			-----------------------------------------
// 08July10		Production Release Version 3.0
//
// SVN Revision Information:
// SVN $Revision: 13654 $
// SVN $Date: 2010-07-08 15:49:19 -0700 (Thu, 08 Jul 2010) $
//
// Resolved SARs
// SAR      Date     Who   Description
//
// Notes:
// best viewed with tabstops set to "4", all port signals are
//				active high, unless otherwise noted
//
// *********************************************************************/
`timescale 1 ns / 1 ps
module CORESDLC_FIFO
#(
parameter ADDR  = 2
) (
input clk, 
input nreset, 
input[7:0] datai, 
input rd, 
input wr, 
input clear, 
output wire[7:0] datao, 
output wire empty, 
output wire full, 
input[7:0] dib, 
output wire[ADDR - 1:0] addra,
output wire[ADDR - 1:0] addrb,
output wire wea,
output wire enb,
output wire[7:0] doa
);
wire[ADDR - 1:0] rdaddr_c; 
reg[ADDR - 1:0] rdaddr; 
wire[ADDR - 1:0] wraddr_c; 
reg[ADDR - 1:0] wraddr; 
wire iempty_c; 
reg iempty; 
wire ifull_c; 
reg ifull; 
reg[7:0] idatao; 
wire[ADDR - 1:0] fone; 
wire[ADDR - 1:0] fzero; 
assign fone = {ADDR{1'b1}} ;
assign fzero = {ADDR{1'b0}} ;
assign rdaddr_c = (rdaddr == fone - 1'b1) ? fzero : rdaddr + 1'b1;
assign wraddr_c = (wraddr == fone - 1'b1) ? fzero : wraddr + 1'b1;
always @(posedge clk or negedge nreset)
begin : rdaddr_reg_proc
    if (nreset == 1'b0)
    begin
        rdaddr <= {ADDR{1'b0}} ; 
    end
    else
    begin
        if (clear == 1'b1)
        begin
            rdaddr <= {ADDR{1'b0}} ; 
        end
        else if (rd == 1'b1 & iempty == 1'b0)
        begin
            rdaddr <= rdaddr_c ; 
        end 
    end 
end 
always @(posedge clk or negedge nreset)
begin : wraddr_reg_proc
    if (nreset == 1'b0)
    begin
        wraddr <= {ADDR{1'b0}} ; 
    end
    else
    begin
        if (clear == 1'b1)
        begin
            wraddr <= {ADDR{1'b0}} ; 
        end
        else if (wr == 1'b1 & ifull == 1'b0)
        begin
            wraddr <= wraddr_c ; 
        end 
    end 
end 
assign ifull_c =
 (rdaddr == wraddr_c & (wr == 1'b1 & rd == 1'b0)) ? 1'b1 : 
 ((rd == 1'b1 & ifull == 1'b1) | clear == 1'b1)   ? 1'b0 : ifull ;
assign iempty_c = 
 ((rdaddr_c == wraddr & (wr == 1'b0 & rd == 1'b1)) | clear == 1'b1) ? 1'b1 :
 (wr == 1'b1 & iempty == 1'b1) ? 1'b0 : iempty ;
always @(posedge clk or negedge nreset)
begin : fe_reg_proc
    if (nreset == 1'b0)
    begin
        ifull <= 1'b0 ; 
        iempty <= 1'b1 ; 
    end
    else
    begin
        ifull <= ifull_c ; 
        iempty <= iempty_c ; 
    end 
end 
always @(posedge clk or negedge nreset)
begin : idatao_reg_proc
    if (nreset == 1'b0)
    begin
        idatao <= {8{1'b1}} ; 
    end
    else
    begin
        idatao <= dib ; 
    end 
end 
assign full = ifull ;
assign empty = iempty ;
assign datao = idatao ;
assign doa = datai ;
assign enb = 1'b1 ;
assign wea = (wr == 1'b1 & ifull == 1'b0) ? 1'b1 : 1'b0 ;
assign addra = wraddr ;
assign addrb = rdaddr ;
endmodule
